@@ -1,6 +1,7 @@
 # coded by lagcleaner
 # email: lagcleaner@gmail.com
 
+# pylint: disable=too-many-lines
 """ Transfer Protocol API Base implementation. """
 
 import datetime as dt
@@ -10,48 +11,51 @@ from multiprocessing import Condition
 from typing import Callable, Union
 
 from multipledispatch import dispatch
-from tfprotocol_client.connection.protocol_client import ProtocolClient
 
-from tfprotocol_client.security.hash_utils import hexstr, sha256_for
 from tfprotocol_client.connection.codes_sender_recvr import CodesSenderRecvr
 from tfprotocol_client.misc.build_utils import MessageUtils
 from tfprotocol_client.misc.constants import (
     BYTE_SIZE,
     DFLT_MAX_BUFFER_SIZE,
+    EMPTY_HANDLER,
     INT_SIZE,
     KEY_LEN_INTERVAL,
     LONG_SIZE,
-    EMPTY_HANDLER,
 )
-from tfprotocol_client.misc.file_stat import FileStat, FileStatTypeEnum
-from tfprotocol_client.misc.status_server_code import StatusServerCode
+from tfprotocol_client.misc.handlers_aliases import (
+    ResponseHandler,
+    SendRecvFileHandler,
+    TransferAsyncHandler,
+    TransferHandler,
+)
 from tfprotocol_client.misc.thread import TfThread
 from tfprotocol_client.models.exceptions import ErrorCode, TfException
+from tfprotocol_client.models.file_stat import FileStat, FileStatTypeEnum
 from tfprotocol_client.models.message import TfProtocolMessage
 from tfprotocol_client.models.proxy_options import ProxyOptions
 from tfprotocol_client.models.putget_commands import PutGetCommandEnum
 from tfprotocol_client.models.status_info import StatusInfo
+from tfprotocol_client.models.status_server_code import StatusServerCode
 from tfprotocol_client.models.transfer_state import TransferStatus
 from tfprotocol_client.tfprotocol_super import TfProtocolSuper
 
-ResponseHandler = Callable[[StatusInfo], None]
-SendRecvFileHandler = Callable[[bool, str, StatusInfo, BytesIO], None]
-TransferHandler = Callable[[ProtocolClient, TransferStatus], None]
-TransferAsyncHandler = Callable[[CodesSenderRecvr], None]
 Date = dt.date
 
 
 class TfProtocol(TfProtocolSuper):
-    """ Tranference Protocol API easy to use.
-        `TfProtocolSuper`: The Abstract mother class of Tranference Protocol API.
+    """Tranference Protocol API easy to use.
+    `TfProtocolSuper`: The Abstract mother class of Tranference Protocol API.
     """
 
     # pylint: disable=super-init-not-called
     @dispatch(TfProtocolSuper, verbosity_mode=False)
     def __init__(
-        self, tfprotocol: TfProtocolSuper, verbosity_mode: bool = False, **_,
+        self,
+        tfprotocol: TfProtocolSuper,
+        verbosity_mode: bool = False,
+        **_,
     ) -> None:
-        """ Constructor for Transfer Protocol class.
+        """Constructor for Transfer Protocol class.
 
         Args:
             `tfprotocol` (TfProtocolSuper): A created transfer protocol instance.
@@ -85,7 +89,7 @@ class TfProtocol(TfProtocolSuper):
         verbosity_mode=False,
         **_,
     ) -> None:
-        """ Constructor for Transfer Protocol class.
+        """Constructor for Transfer Protocol class.
 
         Args:
             `protocol_version` (str, optional): The desired version of the protocol.
@@ -302,7 +306,8 @@ class TfProtocol(TfProtocolSuper):
         timestamp = int(get_time_value.total_seconds() * 1000)
 
         response_handler(
-            timestamp, self.client.translate(TfProtocolMessage('FTOD', formatted_date)),
+            timestamp,
+            self.client.translate(TfProtocolMessage('FTOD', formatted_date)),
         )
 
     def fstat_command(
@@ -326,7 +331,8 @@ class TfProtocol(TfProtocolSuper):
             response_handler(
                 FileStat(
                     FileStatTypeEnum.from_char(
-                        raw_filestat[0][0], dflt=FileStatTypeEnum.UNKNOWN,
+                        raw_filestat[0][0],
+                        dflt=FileStatTypeEnum.UNKNOWN,
                     ),
                     int(raw_filestat[1]),
                     int(raw_filestat[2]),
@@ -338,7 +344,8 @@ class TfProtocol(TfProtocolSuper):
             )
         else:
             response_handler(
-                FileStat(FileStatTypeEnum.UNKNOWN, 0, 0, 0), response,
+                FileStat(FileStatTypeEnum.UNKNOWN, 0, 0, 0),
+                response,
             )
 
     def fupd_command(
@@ -464,13 +471,19 @@ class TfProtocol(TfProtocolSuper):
         response_handler(
             self.client.translate(
                 TfProtocolMessage(
-                    'XCPDIR', new_directory, path, '|', destination_pattern,
+                    'XCPDIR',
+                    new_directory,
+                    path,
+                    '|',
+                    destination_pattern,
                 )
             )
         )
 
     def lock_command(
-        self, lock_filename: str, response_handler: ResponseHandler = EMPTY_HANDLER,
+        self,
+        lock_filename: str,
+        response_handler: ResponseHandler = EMPTY_HANDLER,
     ):
         """Specifies a filename that when it exists in a directory no operation can be done on it,
         except to delete the lock file. This allows a temporary blocking of a directory for any
@@ -486,7 +499,7 @@ class TfProtocol(TfProtocolSuper):
             self.client.translate(TfProtocolMessage('LOCK', lock_filename))
         )
 
-    def sendfile_command(
+    def sndfile_command(
         self,
         is_overriten: bool,
         path: str,
@@ -507,7 +520,10 @@ class TfProtocol(TfProtocolSuper):
             TfProtocolMessage('SNDFILE', '1' if (is_overriten) else '0', path),
         )
         handler(
-            is_overriten, path, response, stream,
+            is_overriten,
+            path,
+            response,
+            stream,
         )
         while True:
             payload = stream.read(
@@ -534,7 +550,7 @@ class TfProtocol(TfProtocolSuper):
             # BREAK IF BAD RESPONSE RECEIVED
             if response.status != StatusServerCode.CONT:
                 break
-        handler(is_overriten, path, self.client.translate('OK'), payload)
+        handler(is_overriten, path, self.client.translate('OK'), stream)
 
     def rcvfile_command(
         self,
@@ -550,7 +566,7 @@ class TfProtocol(TfProtocolSuper):
                 true and tells the server whether the file must be deleted after successfully
                 received by the client.
             `path` (str): The path to the file in the server to be retrieved.
-            `sink` (BytesIO): The sink to send data in byte mode.
+            `sink` (BytesIO): The sink to receive data in byte mode.
             `handler` (SendRecvFileHandler): The function to handle receiving states.
         """
         response = self.client.translate(
@@ -747,7 +763,8 @@ class TfProtocol(TfProtocolSuper):
             `canpt` (int): Cancellation Points, determines the cancellation points where the
                 user, it will have to read from the server to know if it continues or not.
             `response_handler` (ResponseHandler): The function to handle the command response.
-            `transference_handler` (TransferenceHandler): The function to handle the transference cancelations.
+            `transference_handler` (TransferenceHandler): The function to handle the transference
+                cancelations.
         """
         offset, canpt = max(0, offset), max(0, canpt)
 
@@ -759,28 +776,31 @@ class TfProtocol(TfProtocolSuper):
             .add(buffer_size, size=LONG_SIZE, signed=True)
             .add(canpt, size=LONG_SIZE, signed=False)
         )
-        response_handler(response)
 
         # INITIALIZE TRANSFER VARIABLES
         transfer_status = TransferStatus()
         header_size = LONG_SIZE
         server_buffer_size = buffer_size
-        transfer_status.command = PutGetCommandEnum.HPFCONT
+        transfer_status.server_command = PutGetCommandEnum.HPFCONT.value
         if response is None or response.code != 0:
             return
         server_buffer_size = MessageUtils.decode_int(response.payload, signed=True)
+
+        response.code = server_buffer_size
+        response_handler(response)
         i = 0
         while True:
             if canpt > 0 and i == canpt:
                 i = 0
                 cur_header = self.client.just_recv_int(size=header_size, signed=True)
-                transfer_status.dummy_state = True
+                transfer_status.server_command = None
+                transfer_status.handling_canpt = True
                 if cur_header == PutGetCommandEnum.HPFCANCEL.value:
-                    transfer_status.command = PutGetCommandEnum.HPFCANCEL.value
+                    transfer_status.server_command = PutGetCommandEnum.HPFCANCEL.value
                     transfer_handler(self.client, transfer_status)
                     break
                 elif cur_header == PutGetCommandEnum.HPFCONT.value:
-                    transfer_status.command = PutGetCommandEnum.HPFCONT.value
+                    transfer_status.server_command = PutGetCommandEnum.HPFCONT.value
                     transfer_handler(self.client, transfer_status)
                 else:
                     transfer_handler(self.client, transfer_status)
@@ -789,7 +809,7 @@ class TfProtocol(TfProtocolSuper):
                         code=ErrorCode.CAN_PUT,
                         message="Some error ocurred while trying to handle canpt",
                     )
-                transfer_status.dummy_state = False
+                transfer_status.handling_canpt = False
                 continue
             i += 1
             # LOAD DATA TO BE SENT
@@ -802,25 +822,30 @@ class TfProtocol(TfProtocolSuper):
             # SEND DATA CHUNK
             if payl:
                 self.client.send(payl, header_size=LONG_SIZE)
-            transfer_status.dummy = len(payl)
+            transfer_status.last_payload_size = len(payl)
             transfer_handler(self.client, transfer_status)
 
             # HANDLER SIGNAL (SEND HPFCANCEL or HPFSTOP)
-            if transfer_status.dummy in (
+            if transfer_status.client_command in (
                 PutGetCommandEnum.HPFCANCEL.value,
                 PutGetCommandEnum.HPFSTOP.value,
             ):
                 self.client.just_send(
-                    transfer_status.dummy, size=header_size, signed=True
+                    transfer_status.client_command, size=header_size, signed=True
                 )
 
             # BREAK STOP THE CYCLE IF THERE IS NO DATA LEFT IN THE 'data_stream'
             try:
-                if not payl or transfer_status.dummy == PutGetCommandEnum.HPFEND.value:
+                if (
+                    not payl
+                    or transfer_status.client_command == PutGetCommandEnum.HPFEND.value
+                ):
                     self.client.just_send(
-                        PutGetCommandEnum.HPFEND.value, size=header_size, signed=True,
+                        PutGetCommandEnum.HPFEND.value,
+                        size=header_size,
+                        signed=True,
                     )
-                    transfer_status.dummy = PutGetCommandEnum.HPFEND.value
+                    transfer_status.client_command = PutGetCommandEnum.HPFEND.value
                     transfer_handler(self.client, transfer_status)
                     break
             except IOError as e:
@@ -860,31 +885,34 @@ class TfProtocol(TfProtocolSuper):
             .add(buffer_size, size=LONG_SIZE, signed=True)
             .add(canpt, size=LONG_SIZE, signed=False)
         )
-        response_handler(response)
 
         # INITIALIZE TRANSFER VARIABLES
         transfer_status = TransferStatus()
         header_size = LONG_SIZE
         if response is None or response.code != 0:
             return
-        _ = MessageUtils.decode_int(response.payload, signed=True)
+        response.code = MessageUtils.decode_int(response.payload, signed=True)
+
+        response_handler(response)
         i = 0
         while True:
             if canpt > 0 and i == canpt:
                 i = 0
-                transfer_status.dummy = None
-                transfer_status.dummy_state = True
+                transfer_status.client_command = PutGetCommandEnum.HPFCONT.value
+                transfer_status.handling_canpt = True
 
                 transfer_handler(self.client, transfer_status)
-                if not transfer_status.dummy:
-                    transfer_status.dummy = PutGetCommandEnum.HPFCONT.value
+                if not transfer_status.client_command:
+                    transfer_status.client_command = PutGetCommandEnum.HPFCONT.value
                 self.client.just_send(
-                    transfer_status.dummy, size=header_size, signed=True,
+                    transfer_status.client_command,
+                    size=header_size,
+                    signed=True,
                 )
 
-                if transfer_status.dummy == PutGetCommandEnum.HPFCANCEL.value:
+                if transfer_status.client_command == PutGetCommandEnum.HPFCANCEL.value:
                     break
-                transfer_status.dummy_state = False
+                transfer_status.handling_canpt = False
                 continue
             i += 1
 
@@ -894,24 +922,24 @@ class TfProtocol(TfProtocolSuper):
                 PutGetCommandEnum.HPFEND.value,
             ):
                 # CANCELATION FROM SERVER OR END OF FILE REACHED
-                transfer_status.dummy = cur_header
+                transfer_status.server_command = cur_header
 
                 # handle hpffin command exchange at the end of getcan command
-                transfer_handler(
-                    self.client, transfer_status,
-                )
+                transfer_handler(self.client, transfer_status)
                 break
+
             pyld = self.client.just_recv(size=cur_header)
             try:
                 data_sink.write(pyld)
             except IOError as e:
                 raise TfException(exception=e)
-            transfer_status.dummy = len(pyld)
+            transfer_status.last_payload_size = len(pyld)
             transfer_handler(self.client, transfer_status)
 
     def end_command(self):
         """Command is sent by the client to the server in order to terminate the TCP connection."""
         self.client.send('END')
+        self.disconnect()
 
     def sha256_command(
         self, path: str, response_handler: ResponseHandler = EMPTY_HANDLER
@@ -932,7 +960,7 @@ class TfProtocol(TfProtocolSuper):
         uses, but not the only one, is to test whether the server or even the socket communication
         line is still opened, in other words: the keepalive mechanism from the client side
         perspective.
-        
+
         Args:
             `response_handler` (ResponseHandler): The function to handle the command response.
         """
@@ -1013,7 +1041,9 @@ class TfProtocol(TfProtocolSuper):
         t_command: TfThread
         try:
             t_handler = TfThread(
-                transfer_handler, cond_lock=Condition(), args=(code_sr,),
+                transfer_handler,
+                cond_lock=Condition(),
+                args=(code_sr,),
             )
             t_command = TfThread(
                 self.__get_command_t,
@@ -1138,7 +1168,13 @@ class TfProtocol(TfProtocolSuper):
             t_command = TfThread(
                 self.__put_command_t,
                 cond_lock=cond_lock,
-                args=(data_stream, response.code, code_sr),
+                args=(
+                    data_stream,
+                    response.code,
+                    code_sr,
+                    response_handler,
+                    transfer_handler,
+                ),
             )
         except Exception as e:
             raise TfException(
@@ -1267,7 +1303,7 @@ class TfProtocol(TfProtocolSuper):
         this command -in 'advisory' way- to retrieve other servers in order to balance the
         overall traffic. By 'advisory' we mean that it's up to the clients to agree in asking for
         a server from the pool before start any further interaction.
-        
+
         Args:
             `response_handler` (ResponseHandler): The function to handle the command response.
         """
@@ -1488,13 +1524,14 @@ class TfProtocol(TfProtocolSuper):
             )
 
     def fstatls_command(
-        self, path: str, response_handler: ResponseHandler = EMPTY_HANDLER
+        self, path: str, response_handler: Callable[[FileStat], None] = EMPTY_HANDLER
     ):
         """Returns a special structure per each line in the file indicated by the first parameter.
 
         Args:
             `path` (str): Path to file containing the paths to files to get stats.
-            `response_handler` (ResponseHandler): The function to handle the command response.
+            `response_handler` (Callable[[FileStat], None]): The function to handle the command
+                response.
         """
         self.client.send(TfProtocolMessage('FSTATLS', path))
         _code = 0
@@ -1532,7 +1569,7 @@ class TfProtocol(TfProtocolSuper):
     def startnfy_command(
         self,
         interval: Union[float, str, int],
-        response_handler: ResponseHandler = EMPTY_HANDLER,
+        response_handler: Callable[[StatusInfo, Callable, Callable], None] = EMPTY_HANDLER,
     ):
         """Starts the notification system. Starts the watchdog to the list of directory specified
         by the ADDNTFY command. Once STARTNTFY is issued there is no way back to the standard mode.
